import {createStyles, Divider, makeStyles, Theme, Typography} from "@material-ui/core";
import {t, Trans} from "@lingui/macro";
import * as React from "react";
import {observer} from "mobx-react-lite";
import {
    ButtonField,
    Conditional,
    IfFalse,
    IfTrue,
    InputField,
    ModelInput,
    ModelProvider,
    SelectField
} from "@rx-suite/ui";
import {Model} from "@rx-suite/data";
import {red} from "@material-ui/core/colors";
import {LabLayoutComponentModel} from "../../stores/LabLayoutComponentStore";
import {VisualAlertSettings} from "./VisualAlertSettings";
import {AlertGroupSelect} from "./AlertGroupSelect";
import {LayoutEditViewRenderer} from "../../stores/object/LayoutEditViewRenderer";

export type LayoutConfigComponentDetailsProps = {
    renderer: LayoutEditViewRenderer;
};

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        divider: {
            marginTop: theme.spacing(1),
            marginBottom: theme.spacing(1)
        },
        delete: {
            backgroundColor: red[500]
        }
    })
);

export const LayoutConfigComponentDetails: React.FunctionComponent<LayoutConfigComponentDetailsProps> = observer(({renderer}) => {
    const model = renderer.selectedComponent;
    const classes = useStyles();

    const componentSelectOptions = React.useCallback(() => {
        if (!renderer.layout) {
            return [];
        }
        return renderer.layout.current_components.map(c => {
            return {label: c.name, value: c}
        });
    }, [renderer.layout]);

    const handleComponentSelect = React.useCallback((n: string | undefined, component: LabLayoutComponentModel) => {
        renderer.setSelectedComponent(component);
    }, [renderer]);

    const handleDeletedSelected = React.useCallback(() => {
        renderer.deleteSelectedComponent();
    }, [renderer]);

    const handleStatusDotWidthChange = React.useCallback((e) => {
        if (!model) return;
        const val = Number(e.target.value);
        model.viewModel.width = val;
        model.viewModel.height = val;
    }, [model]);

    return (
        <React.Fragment>
            <ModelProvider value={model as Model}>
                <SelectField placeholder={t`Select component...`} onChange={handleComponentSelect}
                             value={model ? model : ""} options={componentSelectOptions()}/>
                <Divider className={classes.divider}/>
                <Conditional condition={!model}>
                    <IfTrue>
                        <Typography variant={'h6'}><Trans>No component selected.</Trans></Typography>
                    </IfTrue>
                    <IfFalse>
                        <ModelInput name={'x'}/>
                        <ModelInput name={'y'}/>
                        <Conditional condition={model?.viewModel.component_type === "machine_status_dot"}>
                            <IfTrue>
                                <InputField label={'Width'} onChange={handleStatusDotWidthChange}
                                            value={model ? model.viewModel.width : ''}/>
                            </IfTrue>
                            <IfFalse>
                                <ModelInput name={'width'}/>
                                <ModelInput name={'height'}/>
                            </IfFalse>
                        </Conditional>
                        <ModelInput label={t`Layer`} name={'z_index'} type={'number'} inputProps={{min: 0, step: 1}}
                                    hint={{
                                        title: t`Layer`,
                                        text: t`Order in which components are placed on the layout. A component with greater layer is always in front of an element with a lower layer.`
                                    }}/>
                        <VisualAlertSettings model={model}/>
                        <AlertGroupSelect model={model}/>
                        <Divider className={classes.divider}/>
                        <ButtonField text={t`Delete Component`} className={classes.delete}
                                     onClick={handleDeletedSelected}/>
                    </IfFalse>
                </Conditional>
            </ModelProvider>
        </React.Fragment>
    )
});