import {attribute, DataStore, model, Model} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from 'yup';
import {action, computed, override} from "mobx";
import {LabLayoutComponentModel} from "./LabLayoutComponentStore";
import {Types} from "@rx-suite/data/lib/Model";

@model({
    transportLayer: "/api/lab_layouts",
    permissionTarget: "LabLayout",
    routingInfo: {
        create: "/lab_layouts/create",
        update: "/lab_layouts/:id/edit",
        read: "/lab_layouts/:id/read"
    }
})

export class LabLayoutModel extends Model {
    constructor() {
        super();
        super.setup();
    }

    static types: Types = {
        components: 'array'
    };

    static relationships = {
        components: LabLayoutComponentModel
    };

    @attribute({
        label: t`id`,
        validation: yup.number()
    })
    id = -1;

    @attribute({
        label: t`Name`,
        validation: yup.string()
    })
    name = "";

    @attribute({
        label: t`Components`,
        validation: yup.array()
    })
    components: LabLayoutComponentModel[] = [];

    @computed
    get current_components(): LabLayoutComponentModel[] {
        return this.components.map(c => c.viewModel);
    }

    @computed
    get current_ordered_component(): LabLayoutComponentModel[] {
        // Return the components ordered by z-index so that selection is always from the top
        return this.components.map(c => c.viewModel).sort((a, b) => a.z_index - b.z_index);
    }

    @computed
    get component_count(): number {
        return this.components.length;
    }

    @action.bound
    deleteComponent(component: LabLayoutComponentModel) {
        this.components.splice(this.current_components.map(c => c._tmpId).indexOf(component._tmpId), 1);
    }

    /* Since the layout form includes files, we have to provide a custom commit */
    @override
    async commit(): Promise<void> {
        //save all the component models
        this.components.forEach(c => c.viewModel.submit());
        this.viewModel.submit();

        const formData = new FormData();
        formData.append('layout', JSON.stringify(this.asJson));
        this.current_components.filter(c => c.component_type == "image").forEach(i => {
            if (i.file) {
                formData.append(i.name, i.file as Blob);
            }
        });
        const response = await LabLayoutStore.transportLayer.fetch(`/save`, {
            method: 'POST',
            body: formData
        })
        return LabLayoutStore.transportLayer.parseJson(response);
    }
}

export const LabLayoutStore = LabLayoutModel.store as DataStore<LabLayoutModel>;

